local registry = LibStub('AceConfigRegistry-3.0')
local optionsTitle = " |cff00ccffWhat's Next?|r"
local main
WhatsNext.optionsClass = "WARRIOR"
WhatsNext.optionsSpec = 3

function DeepCopy(object)
    local lookup_table = {}
    local function _copy(object)
        if type(object) ~= "table" then
            return object
        elseif lookup_table[object] then
            return lookup_table[object]
        end
        local new_table = {}
        lookup_table[object] = new_table
        for index, value in pairs(object) do
            new_table[_copy(index)] = _copy(value)
        end
        return new_table
    end
    return _copy(object)
end

local function GetDefaults()	
	return {
		global = {
			minimap = {
				hide = false,
			},
			placeHolders = {
			},
		},
		profile = {	
			enabled = true,
			locked = false,
			timeBetweenUpdates = .4,
			hideOutOfCombat = false,
			pos = {
				p1 = "TOPLEFT",
				p2 = "CENTER",
				x = 0,
				y = 0,
			},
			configPos = {
				p1 = "CENTER",
				p2 = "CENTER",
				x = 0,
				y = 0,
			}
		}
	}
end

local function SetValue(info,value) 
	WhatsNext.db.profile[info.arg] = value
	WhatsNext:ApplyMainSettings()
end

local function GetValue(info) 
	return WhatsNext.db.profile[info.arg] 
end
			
local optionsTop = {name = "",type = 'group',desc = "",args = {}}

local optionsGeneral = {
	name = "What's Next? by Strucker",
	type = 'group',
	desc = "",
	set = 	SetValue,
	get = 	GetValue,
	handler = WhatsNext,
	args = {
		["ENABLE"] = {
			name = "Enabled",
			type = 'toggle',
			width  = 'full',
			order = 50,
			arg = "enabled",
		},
		["LOCK"] = {
			name = "Locked",
			type = 'toggle',
			width  = 'full',
			order = 150,
			arg = "locked",
		},
		["HIDEOUTOFCOMBAT"] = {
			name = "Hide Out Of Combat",
			type = 'toggle',
			width  = 'full',
			order = 200,
			arg = "hideOutOfCombat",
		},
		["TIME_BETWEEN_UPDATES"] = {
			name = "Seconds Between Updates",
			type = 'range',
			min = .01, max = 1, step = .01,
			order = 250,
			arg = "timeBetweenUpdates"
		},
	}
}

local function OpenCastLogicDisplay(info)
	InterfaceOptionsFrame:Hide()
	WhatsNext_CastLogicDisplay:SetCastLogicSet(WhatsNext.optionsClass,WhatsNext.optionsSpec)

end

local function GetClassList()
	local classList = {}
	classList["DEATHKNIGHT"] = WhatsNext:Localize("DEATHKNIGHT")
	classList["DRUID"] = WhatsNext:Localize("DRUID")
	classList["HUNTER"] = WhatsNext:Localize("HUNTER")
	classList["MAGE"] = WhatsNext:Localize("MAGE")
	classList["MONK"] = WhatsNext:Localize("MONK")
	classList["PALADIN"] = WhatsNext:Localize("PALADIN")
	classList["PRIEST"] = WhatsNext:Localize("PRIEST")
	classList["ROGUE"] = WhatsNext:Localize("ROGUE")
	classList["SHAMAN"] = WhatsNext:Localize("SHAMAN")
	classList["WARLOCK"] = WhatsNext:Localize("WARLOCK")
	classList["WARRIOR"] = WhatsNext:Localize("WARRIOR")
	return classList
end

local function GetSpecList()
	local specList = {}
	specList[1] = WhatsNext:Localize(WhatsNext.optionsClass..1)
	specList[2] = WhatsNext:Localize(WhatsNext.optionsClass..2)
	specList[3] = WhatsNext:Localize(WhatsNext.optionsClass..3)
	if WhatsNext.optionsClass == "DRUID" then
		specList[4] = WhatsNext:Localize(WhatsNext.optionsClass..4)
	end
	return specList
end

local function SetClass(info,value)
	WhatsNext.optionsClass = value
	WhatsNext.optionsSpec = WhatsNext.optionsClass == "DRUID" and WhatsNext.optionsSpec or math.min(WhatsNext.optionsSpec,3)
end

local function GetCastLogicOptions()
	return {
		name = "What's Next? by Strucker",
		type = 'group',
		desc = "",
		set = 	SetValue,
		get = 	GetValue,
		handler = WhatsNext,
		args = {
			["BREAK3"] = {
				name = "\n ",
				type = 'description',
				order = 10,
			},
			["CLASSSELECT"] = {
				type = "select",
				name = "Class",
				values = GetClassList(),
				set = SetClass,
				get = function(info) return WhatsNext.optionsClass end,
				order = 100,
			},
			["BREAK4"] = {
				name = "",
				type = 'description',
				order = 200,
			},
			["SPECSELECT"] = {
				type = "select",
				name = "Specialization",
				values = GetSpecList,
				set = function(info,val) WhatsNext.optionsSpec = val end,
				get = function(info) return WhatsNext.optionsSpec end,
				order = 300,
			},
			["BREAK5"] = {
				name = "\n ",
				type = 'description',
				order = 400,
			},
			["EDIT"] = {
				name = "Edit",
				func = OpenCastLogicDisplay,
				type = 'execute',
				order = 550,
				arg = nil,
			},
		}
	}
end
function WhatsNext:LoadProfile()
	if not self.db.profile.castSequences then
		self.db.profile.castSequences = DeepCopy(self.defaultCastSequences)
	end
	self:ApplyMainSettings()
end

function WhatsNext:RegisterOptions()
	local dialog = LibStub('AceConfigDialog-3.0')
	
	self.db = LibStub('AceDB-3.0'):New('WhatsNextDB', GetDefaults()	, 'Default')
	self.db.RegisterCallback(self, "OnProfileChanged", "LoadProfile")
	self.db.RegisterCallback(self, "OnProfileCopied", "LoadProfile")
	self.db.RegisterCallback(self, "OnProfileReset", "LoadProfile")
	self:LoadProfile()
	
	
	local profileOptions = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	registry:RegisterOptionsTable("WhatsNext", {name = "",type = 'group',desc = "",args = {}})
	local top = dialog:AddToBlizOptions("WhatsNext", optionsTitle)
	top:SetScript('OnShow', function(self)
		InterfaceOptionsFrame_OpenToCategory(main)
	end)
	
	registry:RegisterOptionsTable("WhatsNext-General", optionsGeneral)
	main = dialog:AddToBlizOptions("WhatsNext-General", "General", optionsTitle)
	
	registry:RegisterOptionsTable("WhatsNext-CastLogic", GetCastLogicOptions())
	WhatsNext.castLogicOptions = dialog:AddToBlizOptions("WhatsNext-CastLogic", "Logic Sets", optionsTitle)
	
	registry:RegisterOptionsTable("WhatsNext-Profile", profileOptions)
	dialog:AddToBlizOptions("WhatsNext-Profile", "Profile", optionsTitle)
	
	local dataobj = LibStub:GetLibrary('LibDataBroker-1.1'):NewDataObject("What's Next?", {
		type = 'launcher',
		text = "What's Next?",
		label = "What's Next?",
		icon = 'Interface\\Icons\\INV_Misc_QuestionMark',
	})
	
	if dataobj then
		dataobj.OnClick = function(self,button,down) 
			if button == "LeftButton" then
				WhatsNext:ShowOptions()
			else
				WhatsNext.db.profile.locked = not WhatsNext.db.profile.locked
				WhatsNext:ApplyMainSettings()
			end
		end
		dataobj.OnTooltipShow = function(tooltip)
			tooltip:AddLine("|cff00ccffWhat's Next?|r by Strucker")
			tooltip:AddLine("     |cffffffffLeft Click|r -> Toggle Options Display")
			tooltip:AddLine("     |cffffffffRight Click|r -> Toggle Lock")
		end
		
		LibStub("LibDBIcon-1.0", true):Register("What's Next?", dataobj, self.db.global.minimap)
	end
end

function WhatsNext:ShowOptions()
	if InterfaceOptionsFrame:IsVisible() then
		InterfaceOptionsFrame:Hide()
	else
		InterfaceOptionsFrame_OpenToCategory(main) 
	end
end